@@ -1,8 +1,8 @@
 # coding: utf-8
 
 # general imports
-import os, re
-from time import gmtime, strftime
+import os
+import re
 
 # django imports
 from django.conf import settings as django_settings
@@ -25,13 +25,12 @@ from django.views.decorators.csrf import csrf_exempt
 
 # filebrowser imports
 from filebrowser_safe.settings import *
-from filebrowser_safe.functions import (path_to_url, sort_by_attr, get_path,
-                get_file, get_breadcrumbs, get_filterdate, get_settings_var,
-                convert_filename)
+from filebrowser_safe.functions import (sort_by_attr, get_path,
+                get_breadcrumbs, get_filterdate, get_settings_var,
+                get_directory, convert_filename)
 from filebrowser_safe.templatetags.fb_tags import query_helper
 from filebrowser_safe.base import FileObject
 from filebrowser_safe.decorators import flash_login_required
-from filebrowser_safe.storage import FileSystemStorageMixin
 
 from mezzanine.utils.importing import import_dotted_path
 
@@ -52,8 +51,8 @@ else:
 # Precompile regular expressions
 filter_re = []
 for exp in EXCLUDE:
-   filter_re.append(re.compile(exp))
-for k,v in VERSIONS.iteritems():
+    filter_re.append(re.compile(exp))
+for k, v in VERSIONS.iteritems():
     exp = (r'_%s.(%s)') % (k, '|'.join(EXTENSION_LIST))
     filter_re.append(re.compile(exp))
 
@@ -72,16 +71,16 @@ def browse(request):
         msg = _('The requested Folder does not exist.')
         messages.add_message(request, messages.ERROR, msg)
         if directory is None:
-            # The DIRECTORY does not exist, raise an error to prevent eternal redirecting.
+            # The directory returned by get_directory() does not exist, raise an error to prevent eternal redirecting.
             raise ImproperlyConfigured, _("Error finding Upload-Folder. Maybe it does not exist?")
         redirect_url = reverse("fb_browse") + query_helper(query, "", "dir")
         return HttpResponseRedirect(redirect_url)
-    abs_path = os.path.join(DIRECTORY, path)
+    abs_path = os.path.join(get_directory(), path)
 
     # INITIAL VARIABLES
-    results_var = {'results_total': 0, 'results_current': 0, 'delete_total': 0, 'images_total': 0, 'select_total': 0 }
+    results_var = {'results_total': 0, 'results_current': 0, 'delete_total': 0, 'images_total': 0, 'select_total': 0}
     counter = {}
-    for k,v in EXTENSIONS.iteritems():
+    for k, v in EXTENSIONS.iteritems():
         counter[k] = 0
 
     dir_list, file_list = default_storage.listdir(abs_path)
@@ -98,7 +97,9 @@ def browse(request):
         results_var['results_total'] += 1
 
         # CREATE FILEOBJECT
-        fileobject = FileObject(os.path.join(DIRECTORY, path, file))
+        fileobject = FileObject(os.path.join(get_directory(), path, file))
+        # Strip leading slash in dirnames for MEDIA_LIBRARY_PER_SITE
+        fileobject.path_relative_directory = fileobject.path_relative_directory.lstrip("/")
 
         # FILTER / SEARCH
         append = False
@@ -168,6 +169,7 @@ browse = staff_member_required(never_cache(browse))
 filebrowser_pre_createdir = Signal(providing_args=["path", "dirname"])
 filebrowser_post_createdir = Signal(providing_args=["path", "dirname"])
 
+
 def mkdir(request):
     """
     Make Directory.
@@ -182,7 +184,7 @@ def mkdir(request):
         msg = _('The requested Folder does not exist.')
         messages.add_message(request, messages.ERROR, msg)
         return HttpResponseRedirect(reverse("fb_browse"))
-    abs_path = os.path.join(DIRECTORY, path)
+    abs_path = os.path.join(get_directory(), path)
 
     if request.method == 'POST':
         form = MakeDirForm(abs_path, request.POST)
@@ -252,6 +254,7 @@ def upload(request):
     }, context_instance=Context(request))
 upload = staff_member_required(never_cache(upload))
 
+
 @csrf_exempt
 def _check_file(request):
     """
@@ -262,9 +265,9 @@ def _check_file(request):
     folder = fb_uploadurl_re.sub('', folder)
     fileArray = {}
     if request.method == 'POST':
-        for k,v in request.POST.items():
+        for k, v in request.POST.items():
             if k != "folder":
-                if default_storage.exists(os.path.join(DIRECTORY, folder, v)):
+                if default_storage.exists(os.path.join(get_directory(), folder, v)):
                     fileArray[k] = v
     return HttpResponse(simplejson.dumps(fileArray))
 
@@ -273,6 +276,7 @@ def _check_file(request):
 filebrowser_pre_upload = Signal(providing_args=["path", "file"])
 filebrowser_post_upload = Signal(providing_args=["path", "file"])
 
+
 @csrf_exempt
 @flash_login_required
 @staff_member_required
@@ -289,17 +293,17 @@ def _upload_file(request):
 
         if request.FILES:
             filedata = request.FILES['Filedata']
-            abs_path = os.path.join(DIRECTORY, folder, filedata.name)
             # PRE UPLOAD SIGNAL
             filebrowser_pre_upload.send(sender=request, path=request.POST.get('folder'), file=filedata)
 
             filedata.name = convert_filename(filedata.name)
 
             # HANDLE UPLOAD
-            exists = default_storage.exists(os.path.join(DIRECTORY, folder, filedata.name))
+            exists = default_storage.exists(os.path.join(get_directory(), folder, filedata.name))
+            abs_path = os.path.join(get_directory(), folder, filedata.name)
             uploadedfile = default_storage.save(abs_path, filedata)
 
-            path = os.path.join(DIRECTORY, folder)
+            path = os.path.join(get_directory(), folder)
             file_name = os.path.join(path, filedata.name)
             if exists:
                 default_storage.move(smart_unicode(uploadedfile), smart_unicode(file_name), allow_overwrite=True)
@@ -313,6 +317,7 @@ def _upload_file(request):
 filebrowser_pre_delete = Signal(providing_args=["path", "filename"])
 filebrowser_post_delete = Signal(providing_args=["path", "filename"])
 
+
 def delete(request):
     """
     Delete existing File/Directory.
@@ -334,9 +339,9 @@ def delete(request):
             msg = _('The requested File does not exist.')
         messages.add_message(request, messages.ERROR, msg)
         return HttpResponseRedirect(reverse("fb_browse"))
-    abs_path = os.path.join(DIRECTORY, path)
+    abs_path = os.path.join(get_directory(), path)
     if request.GET.get('filetype') != "Folder":
-        relative_server_path = os.path.join(DIRECTORY, path, filename)
+        relative_server_path = os.path.join(get_directory(), path, filename)
         try:
             # PRE DELETE SIGNAL
             filebrowser_pre_delete.send(sender=request, path=path, filename=filename)
@@ -373,6 +378,7 @@ delete = staff_member_required(never_cache(delete))
 filebrowser_pre_rename = Signal(providing_args=["path", "filename", "new_filename"])
 filebrowser_post_rename = Signal(providing_args=["path", "filename", "new_filename"])
 
+
 def rename(request):
     """
     Rename existing File/Directory.
@@ -393,15 +399,15 @@ def rename(request):
             msg = _('The requested File does not exist.')
         messages.add_message(request, messages.ERROR, msg)
         return HttpResponseRedirect(reverse("fb_browse"))
-    abs_path = os.path.join(MEDIA_ROOT, DIRECTORY, path)
+    abs_path = os.path.join(MEDIA_ROOT, get_directory(), path)
     file_extension = os.path.splitext(filename)[1].lower()
 
     if request.method == 'POST':
         form = RenameForm(abs_path, file_extension, request.POST)
         if form.is_valid():
-            relative_server_path = os.path.join(DIRECTORY, path, filename)
+            relative_server_path = os.path.join(get_directory(), path, filename)
             new_filename = form.cleaned_data['name'] + file_extension
-            new_relative_server_path = os.path.join(DIRECTORY, path, new_filename)
+            new_relative_server_path = os.path.join(get_directory(), path, new_filename)
             try:
                 # PRE RENAME SIGNAL
                 filebrowser_pre_rename.send(sender=request, path=path, filename=filename, new_filename=new_filename)
